#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os
import apt
import apt.progress.base as apb
from apt.debfile import DebPackage
from apt.cache import FetchFailedException
import locale
import re

class AppActions:
    INS_AND_UP = "install_and_upgrade"
    INSTALL = "install"
    INSTALL_ONE = "insone"
    INSTALL_DEBFILE = "install_debfile"
    UPGRADE = "upgrade"
    UPGRADE_ONE = "upone"
    UPDATE = "update"
    UPDATESHELL = "update_by_shell"
    CONFIGURESHELL = "configure_dpkg_by_shell"
    FIXCONFFILE = "fix_conffile_by_shell"


class FetchProcess(apb.AcquireProgress):
    taskPkgs = []

    def __init__(self, dbus_service, appname, action,pkgs):
        apb.AcquireProgress.__init__(self)
        self.dbus_service = dbus_service
        self.appname = appname
        self.action = action
        self.pkgname=''.join(pkgs)

    #
    # 函数:下载结束
    # item:下载的item
    #
    def done(self, item):
        self.percent=0.0
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "action":str(self.action),
                 "download_pkgname": str(self.pkgname),
                 }

        # 一个包下载成功，从任务列表中删除
        onePkgName = item.description.split()[3]
        if(onePkgName in self.taskPkgs):
            self.taskPkgs.remove(onePkgName)

        self.dbus_service.kum_fetch_signal("down_done", kwarg)

    #
    # 函数:下载失败
    # item:下载的item
    #
    def fail(self, item):
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "action":str(self.action),
                 }

        self.dbus_service.kum_fetch_signal("down_fail", kwarg)

    #
    # 函数:接收数据
    # item:下载的item
    #
    def fetch(self, item):
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "action":str(self.action),
                 }

        self.dbus_service.kum_fetch_signal("down_fetch", kwarg)

    #
    # 函数:接触到ims
    # item:下载的item
    #
    def ims_hit(self, item):
        # 'ims_hit'
        pass

    #
    # 函数:媒介切换
    # media:媒介
    #
    def media_change(self, media, drive):
        # 'media_change'
        pass

    #
    # 函数:下载心跳状态
    #
    def pulse(self, owner):

        if self.action == AppActions.UPDATE:
            if self.total_items!= 0:
                percent = float(self.current_items * 100.0 / self.total_items)
                if percent > self.percent:
                    self.percent = percent
        else:
            if self.total_bytes != 0:
                self.percent = float(self.current_bytes * 100.0 / self.total_bytes)

        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "download_bytes":str(self.current_bytes),
                 "total_bytes":str(self.total_bytes),
                 "download_items":str(self.current_items),
                 "total_items":str(self.total_items),
                 "action":str(self.action),
                 "download_pkgname": str(self.pkgname),
                 }

        self.dbus_service.kum_fetch_signal("down_pulse", kwarg)

        # cancel the operation
        if self.dbus_service.check_cancel_worker_item(self.appname) is True:
            # ("download_cancel")
            self.dbus_service.kum_fetch_signal("down_cancel", kwarg)
            return False
    #
    # 函数:开始下载
    #
    def start(self):
        # Reset all our values.
        self.current_bytes = 0.0
        self.current_cps = 0.0
        self.current_items = 0
        self.elapsed_time = 0
        self.fetched_bytes = 0.0
        self.last_bytes = 0.0
        self.total_bytes = 0.0
        self.total_items = 0
        self.percent = 0

        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "download_items":str(self.total_items),
                 "action":str(self.action),
                 "download_pkgname": str(self.pkgname),
                 }

        self.dbus_service.kum_fetch_signal("down_start", kwarg)

    #
    # 函数:下载停止
    #
    def stop(self):
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(100),
                 "action":str(self.action),
                 }

        #有包下载失败但是未触发error，大多是因为依赖不满足导致，手动发送error信号
        # if self.total_bytes == 0.0 or self.current_bytes == 0.0:
        #     kwarg = {"apt_appname": self.appname,
        #              "apt_percent": "-99",
        #              "action": str(self.action),
        #              "status": "deps_error",
        #              }
        #     self.dbus_service.kum_apt_signal("down_error", kwarg)
        # else:
        self.dbus_service.kum_fetch_signal("down_stop", kwarg)


class AptProcess(apb.InstallProgress):
    '''Apt progress'''
    def __init__(self, dbus_service, appname, action,count,pkg_name):
        apb.InstallProgress.__init__(self)
        self.dbus_service = dbus_service
        self.appname = appname
        self.percent = 0
        self.action = action
        self.install_num=count
        self.pkgname=''.join(pkg_name)

    #
    # 函数:冲突处理
    #
    def conffile(self, current, new):
        kwarg = {"apt_appname":str(current),
                 "apt_percent":str(-1),
                 "action":str(new),
                 }
        self.dbus_service.kum_apt_signal("apt_conffile", kwarg)

    #
    # 函数:错误处理
    #
    def error(self, pkg, errormsg):
        kwarg = {"apt_appname":str(pkg),
                 "apt_percent":-99.0,
                 "action":str(self.action),
                 }
        self.dbus_service.kum_apt_signal("apt_error", kwarg)

    #
    # 函数:开始处理
    #
    def start_update(self):
        kwarg = {"apt_appname":self.appname,
                 "apt_percent":str(self.percent),
                 "action":str(self.action),
                 }

        self.dbus_service.kum_apt_signal("apt_start", kwarg)

    #
    # 函数:处理结束
    #
    def finish_update(self):
        pass
        '''
        if self.percent != 0.0:
            pass
            
            kwarg = {"apt_appname":self.appname,
                     "apt_percent":str(100),
                     "action":str(self.action),
                     "install_num": str(self.install_num),
                     "apt_pkgname": str(self.pkgname)
                     }
            self.dbus_service.kum_apt_signal("apt_finish", kwarg)
            
        else:
            kwarg = {"apt_appname": self.appname,
                     "apt_percent": str(-99.0),
                     "action": str(self.action),
                     "install_num": str(self.install_num),
                     "apt_pkgname": str(self.pkgname)
                     }
            self.dbus_service.kum_apt_signal("apt_error", kwarg)
        '''
    #
    # 函数:处理过程
    #
    def status_change(self, pkg, percent, status):
        #print("status change",pkg,percent,status,self.appname)
        kwarg = {"apt_appname":pkg,
                 "apt_percent":str(percent),
                 "action":str(self.action),
                 "status":str(status),
                 }  
        self.dbus_service.kum_apt_signal("apt_pulse", kwarg)
        if re.search("已安装",status) is not None:
            kwarg1 = {"apt_appname":pkg,
                     "apt_percent":str(100),
                     "action":str(self.action),
                     "install_num": str(self.install_num),
                     "apt_pkgname": str(pkg)
                     }
            self.dbus_service.kum_apt_signal("apt_finish", kwarg1)


class AptHandler():

    def __init__(self, dbus_service):
        self.dbus_service = dbus_service
        locale.setlocale(locale.LC_ALL, "zh_CN.UTF-8")
        # self.cache = apt.Cache()
        # self.cache.open()

    #
    # 函数:初始化cache
    #
    def init_cache(self):
        self.cache = apt.Cache()
        self.cache.open()

    #
    # 函数:获取软件包
    # pkgName:软件包名
    #
    def get_pkg_by_name(self, pkgName):
        try:
            return self.cache[pkgName]
        except KeyError:
            raise WorkitemError(1, "Package %s is not  available" % pkgName)


    #
    # 函数:批量自动安装或升级
    # pkg_names:软件包名
    #
    def install_and_upgrade(self, pkg_name, kwargs=None):
        # 删除本地缓存的deb安装包
        #os.popen("rm /var/cache/apt/archives/*.deb").readlines()

        #self.cache.open()
        taskPkgs = []

        #for pkg_name in pkg_names:
        self.cache.open()
        pkg = self.get_pkg_by_name(pkg_name)
        if(pkg.is_installed == False):
            pkg.mark_install()
            taskPkgs.append(pkg.name)
            try:
                self.cache.commit(FetchProcess(self.dbus_service, pkg_name, AppActions.INSTALL_ONE, pkg_name),
                                  AptProcess(self.dbus_service, pkg.name, AppActions.INSTALL_ONE,1,pkg_name))
            except apt.cache.FetchFailedException as error:
                raise WorkitemError(2, str(error))
            except apt.cache.LockFailedException:
                raise WorkitemError(3, "package manager is running.")
            except Exception as e:
                raise WorkitemError(0, "unknown error")

        if(pkg.is_upgradable == True):
            pkg.mark_upgrade()
            taskPkgs.append(pkg.name)
            try:
                fp = FetchProcess(self.dbus_service, pkg_name, AppActions.UPGRADE_ONE, pkg_name)
                fp.taskPkgs = taskPkgs
                self.cache.commit(fp,
                                  AptProcess(self.dbus_service, pkg_name, AppActions.UPGRADE_ONE, 1, pkg_name))
            except apt.cache.FetchFailedException as error:
                raise WorkitemError(2, str(error))
            except apt.cache.LockFailedException:
                raise WorkitemError(3, "package manager is running.")
            except Exception as e:
                raise WorkitemError(0, "unknown error")
            self.cache.close()

    #
    # 函数：批量安装
    # pkgNames:软件包名
    #
    def install(self, pkgNames, kwargs=None):
        self.cache.open()
        count = len(pkgNames)

        for pkgName in pkgNames:
            pkg = self.get_pkg_by_name(pkgName)
            pkg.mark_install()

        try:
            self.cache.commit(FetchProcess(self.dbus_service, "#install", AppActions.INSTALL,pkgNames), AptProcess(self.dbus_service, "#install", AppActions.INSTALL,count,pkgNames))
        except apt.cache.FetchFailedException as error:
            raise WorkitemError(2, str(error))

        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")

        except Exception as e:
            raise WorkitemError(0, "unknown error")

    #
    # 函数：单个安装
    # pkgName:软件包名
    #
    def insone(self, pkgName, kwargs=None):
        self.cache.open()
        pkg = self.get_pkg_by_name(pkgName)
        pkg.mark_install()

        try:
            self.cache.commit(FetchProcess(self.dbus_service, pkgName, AppActions.INSTALL_ONE,pkgName), AptProcess(self.dbus_service, pkgName, AppActions.INSTALL_ONE,1,pkgName))
        except apt.cache.FetchFailedException as error:
            raise WorkitemError(2, str(error))
        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            raise WorkitemError(0, "unknown error")

    #
    # 函数：批量升级
    # pkgNames：软件包名
    #
    def upgrade(self, pkgNames, kwargs=None):
        self.cache.open()
        count = len(pkgNames)

        for pkgName in pkgNames:
            pkg = self.get_pkg_by_name(pkgName)
            pkg.mark_upgrade()

        try:
            self.cache.commit(FetchProcess(self.dbus_service, "#upgrade", AppActions.UPGRADE,pkgNames), AptProcess(self.dbus_service, "#upgrade", AppActions.UPGRADE,count,pkgNames))
        except apt.cache.FetchFailedException as error:
            raise WorkitemError(2, str(error))
        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            raise WorkitemError(0, "unknown error")

    #
    # 函数：单个升级
    # pkgName：软件包名
    #
    def upone(self, pkgName, kwargs=None):
        self.cache.open()
        pkg = self.get_pkg_by_name(pkgName)
        pkg.mark_upgrade()

        try:
            self.cache.commit(FetchProcess(self.dbus_service, pkgName, AppActions.UPGRADE_ONE,pkgName), AptProcess(self.dbus_service, pkgName, AppActions.UPGRADE_ONE,1,pkgName))
        except apt.cache.FetchFailedException as error:
            raise WorkitemError(2, str(error))
        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            raise WorkitemError(0, "unknown error")

    #
    # 函数：调用python-apt库apt-get update
    # taskName：任务
    #
    def update(self, taskName, kwargs=None):
        quiet = False
        if kwargs is not None:
            quiet = int(kwargs["quiet"])

        try:
            if quiet == True:
                print("quiet=True")
                self.cache.update()
            else:
                print("quiet=False")
                self.cache.update(fetch_progress=FetchProcess(self.dbus_service, taskName, AppActions.UPDATE))
        except Exception as e:
            print(e)
            print("update except")

    #
    # 函数：调用本地命令apt-get update
    # taskName：任务
    def update_by_shell(self, taskName, kwargs=None):
        out_msg = os.popen("apt-get update").readlines()
        kwarg = {"apt_appname":str(taskName),
                 "apt_percent":str(200),
                 # "action":"#update_by_shell",
                 "action":str(kwargs["quiet"]),
                 }
        self.dbus_service.kum_apt_signal("finish", kwarg)

    #
    # 函数：调用本地命令修复dpkg
    # 调用本地命令卸载可能存在的SP升级包,默认选择Yes
    # taskName：任务
    #
    def configure_dpkg_by_shell(self, taskName, kwargs=None):
        os.popen("dpkg --configure -a --force-all").readlines()
        #os.popen("dpkg -P --force-all kylin-update-desktop").readlines()
        #os.popen("dpkg -P --force-all kylin-update-server").readlines()

        kwarg = {"apt_appname":str(taskName),
                 "apt_percent":str(200),
                 "action":"#configure_dpkg_by_shell",
                 "status":str("ha"),
                 }

        self.dbus_service.kum_apt_signal("finish", kwarg)

    #
    # 函数：调用本地命令修复conffile冲突，默认选择Yes
    # taskName：任务
    #
    def fix_conffile_by_shell(self, taskName, kwargs=None):
        os.popen("dpkg --configure -a --force-all").readlines()

        kwarg = {"apt_appname":str(taskName),
                 "apt_percent":str(200),
                 "action":"#fix_conffile_by_shell",
                 "status":str("he"),
                 }

        self.dbus_service.kum_apt_signal("finish", kwarg)

    #
    # 函数：安装本地deb包
    #
    def install_debfile(self, path, sender=None):
        # print("####install deb file: ", path)
        # 开启密码认证机制
        if not os.path.isfile(path):
            raise WorkitemError(4, "%s is unreadable file" % path)
        try:
            debfile = DebPackage(path)
        except IOError:
            raise WorkitemError(4, "%s is unreadable file" % path)
        except Exception as e:
            raise WorkitemError(5, e)
        #self.cache.open()
        #获取软件包名
        pkgName = debfile._sections["Package"]
        debfile.check() #do debfile.check for the next to do debfile.missing_deps
        #安装软件包
        res = debfile.install()
        if res:
            kwarg = {"apt_appname": pkgName,
                    "apt_percent": str(-99.0),
                    "action": str(AppActions.INSTALL_DEBFILE),
                    }
            self.dbus_service.software_apt_signal("apt_finish", kwarg)
            raise WorkitemError(6, "package manager failed")
        else:
            kwarg = {"apt_appname": pkgName,
                    "apt_percent": str(100),
                    "action": str(AppActions.INSTALL_DEBFILE),
                    }
            self.dbus_service.kum_apt_signal("apt_finish", kwarg)
        # else:
        #     kwarg = {"apt_appname": pkgName,
        #              "apt_percent": str(-200),
        #              "action": str(AppActions.INSTALLDEBFILE),
        #              }
        #     self.software_apt_signal("apt_error", kwarg)
        #     raise WorkitemError(16, "dependence not be satisfied")
        return True

class WorkitemError(Exception):

    def __init__(self, errornum, details = ""):
        self.errornum = errornum
        self.details = details


if __name__ == "__main__":
    ad = AptHandler(None)
    ad.upgrade(['khelpcenter4'])