#!/usr/bin/python3
# -*- coding: utf-8 -*-

__author__ = 'lixiang'

from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import QWidget, QHBoxLayout, QLabel, QFrame, QApplication

from mainui.systembutton import SystemButton

from utils.enums import VERSION

class TitleBar(QWidget):

    minTitleBar = pyqtSignal(name="minTitleBar")
    closeTitleBar = pyqtSignal(name="closeTitleBar")

    def __init__(self, width, has_btn_set,has_btn_min, title = "", parent=None):
        super(TitleBar, self).__init__(parent)

        self.setFixedSize(width, 40)
        self.setWindowFlags(Qt.FramelessWindowHint)
        #self.setAttribute(Qt.WA_TranslucentBackground)

        self.setAutoFillBackground(True)
        #self.setStyleSheet("border-radius:6px;border:1px solid #cfcfcf;")

        palette = QPalette()
        palette.setColor(self.backgroundRole(), QColor(255,255,255))     # 0f84bc
        self.setPalette(palette)

        self.min_button = SystemButton(False,True,False,self)
        self.min_button.setGeometry(self.width()-68,4,30,30)
        self.set_button = SystemButton(True,False,False,self)
        self.set_button.setGeometry(self.width()-102,4,30,30)
        if has_btn_min:
            self.min_button.show()
        else:
            self.min_button.hide()
        if has_btn_set:
            self.set_button.show()
        else:
            self.set_button.hide()
        self.close_button = SystemButton(False,False,True,self)
        self.close_button.setGeometry(self.width()-34,4,30,30)

        self.logo_label = QLabel()
        self.logo_label.setFixedSize(22, 22)
        self.logo_label.setFrameShape(QFrame.NoFrame)
        self.logo_label.setAutoFillBackground(True)
        self.logo_label.setScaledContents(True)
        #self.label_pixmap = QPixmap("./res/icon.png")
        #self.logo_label.setPixmap(self.label_pixmap)
        self.logo_label.setStyleSheet("QLabel{background-color:white;background-image:url(./res/icon.png)}")
        self.title_label = QLabel()
        self.title_label.setStyleSheet("QLabel{color:#444444;font-size:14px;}")
        self.title_label.setText(title)

        hlayout = QHBoxLayout()
        hlayout.addSpacing(5)
        hlayout.addWidget(self.logo_label, 0, Qt.AlignVCenter)
        hlayout.addSpacing(5)
        hlayout.addWidget(self.title_label, 0, Qt.AlignVCenter)
        hlayout.setSpacing(5)
        hlayout.setContentsMargins(4, 0, 4, 0)


        main_layout = QHBoxLayout()
        main_layout.setSpacing(0)
        main_layout.addLayout(hlayout)
        main_layout.addStretch()
        main_layout.setContentsMargins(0, 0, 0, 0)

        self.setLayout(main_layout)

    '''def paintEvent(self,event):
        painter=QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.setBrush(QBrush(Qt.white))
        painter.setPen(Qt.transparent)
        rect = self.rect()
        rect.setWidth(rect.width())
        rect.setHeight(rect.height())
        painter.drawRoundedRect(rect, 6, 6)'''



if __name__ == '__main__':
    import sys
    app = QApplication(sys.argv)
    #title = TitleBar(280, True)
    #title.show()
    sys.exit(app.exec_())