#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os
import sys
# reload(sys)
# sys.setdefaultencoding('utf8')

import dbus
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop
from kylindbus.daemon import Daemon
from utils.enums import INTERFACE, INTERFACES, Signals, CONF_MODEL_PATH, \
    SOFTWARE_PROPERTIES_INTERFACE, SOFTWARE_PROPERTIES_PATH, SOFTWARE_PROPERTIES_SERVICE
import signal
import dbus.mainloop.glib
from gi.repository import GObject
from PyQt5.QtCore import *
import time
from PyQt5.QtDBus import *

import gettext

mainloop = DBusGMainLoop(set_as_default=True)
gettext.textdomain("kylin-update-manager")
_ = gettext.gettext


class DbusService(QObject, Signals):

    dbusDaemon = None
    sourceDaemon = None
    iface = None
    down_pulse = pyqtSignal(str, str)
    down_finish = pyqtSignal(bool, str, str)
    down_start = pyqtSignal(str, str)
    apt_start = pyqtSignal(str, str)
    apt_finish = pyqtSignal(int, str)
    down_dep_finish = pyqtSignal(str)
    send_progress = pyqtSignal(str)
    deb_copy_finish = pyqtSignal(str)
    # dbus_process_msg = pyqtSignal(name="dbus_process_msg")
    install_num = 0

    def __init__(self):
        QObject.__init__(self)

        # print "正在启动dbus服务..."
        print(_("dbus service starting..."))
        self.init_dbus()

    #
    # 函数：退出dbus
    #
    def exit_dbus(self):
        self.dbusDaemon.exit()

    #
    # 函数：初始化dbus
    #
    def init_dbus(self):
        try:
            bus = dbus.SystemBus(mainloop)
            qdbus = QDBusConnection.systemBus()
        except:
            # print "无法初始化dbus..."
            print(_("dbus initialise failed..."))
            sys.exit(0)

        # dbus已经在运行，直接获取
        try:
            dbusObj = bus.get_object(INTERFACE, INTERFACES, INTERFACE)
            self.dbusDaemon = dbus.Interface(dbusObj, INTERFACE)
            self.dbusDaemon.connect_to_signal("kum_fetch_signal", self.slot_kum_fetch_signal)
            self.dbusDaemon.connect_to_signal("kum_apt_signal", self.slot_kum_apt_signal)
            self.dbusDaemon.connect_to_signal("copy_finish",self.slot_copy_finish)
            self.dbusDaemon.connect_to_signal("down_dep_finish", self.slot_down_dep_finish)
            self.record_open_time(CONF_MODEL_PATH, "last-start-time", "time")
            try:
                sourceDbus = bus.get_object(SOFTWARE_PROPERTIES_SERVICE,SOFTWARE_PROPERTIES_PATH,SOFTWARE_PROPERTIES_INTERFACE)
                self.sourceDaemon = dbus.Interface(sourceDbus, SOFTWARE_PROPERTIES_INTERFACE)
            except:
                sourceDbus = None
            self.iface = QDBusInterface(SOFTWARE_PROPERTIES_SERVICE, SOFTWARE_PROPERTIES_PATH, SOFTWARE_PROPERTIES_INTERFACE, qdbus)
            self.iface.setTimeout(100000000)

        # dbus还未运行，启动一个新的
        except dbus.DBusException:
            os.environ["TERM"] = "xterm"
            os.environ["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/X11R6/bin"
            os.environ["DEBIAN_FRONTEND"] = "noninteractive"
            # if os.path.exists("/var/lib/apt/lists/lock"):
            #     os.remove("/var/lib/apt/lists/lock")
            dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
            GObject.threads_init()
            signal.signal(signal.SIGINT, lambda: mainloop.quit())
            self.dbusDaemon = Daemon(dbus.SystemBus(), mainloop)
            self.dbusDaemon.connect_to_signal("kum_fetch_signal", self.slot_kum_fetch_signal)
            self.dbusDaemon.connect_to_signal("kum_apt_signal", self.slot_kum_apt_signal)
            self.dbusDaemon.connect_to_signal("copy_finish",self.slot_copy_finish)
            self.dbusDaemon.connect_to_signal("down_dep_finish", self.slot_down_dep_finish)
            self.record_open_time(CONF_MODEL_PATH, "last-start-time", "time")
            mainloop.run()

    #
    # 函数：记录当前应用打开时间
    #
    def record_open_time(self, path, group, key):
        lasttime = time.strftime('%Y%m%d%H', time.localtime())
        self.dbusDaemon.modify_conf(path, group, key, lasttime)

    #
    # 函数：调用dbus函数
    #
    def call_init_cache(self):
        self.dbusDaemon.init_cache()

    #
    # 函数：调用dbus函数
    #
    def call_update_source(self):
        self.dbusDaemon.update(False)

    #
    # 函数：调用dbus函数
    #
    def call_update_source_by_shell(self, quiet):
        self.dbusDaemon.update_by_shell(quiet)

    #
    # 函数：调用dbus函数
    #
    def call_configure_dpkg_by_shell(self):
        self.dbusDaemon.configure_dpkg_by_shell(False)

    #
    # 函数：调用dbus函数
    #
    def call_install_and_upgrade(self, pkgName):
        self.dbusDaemon.install_and_upgrade(pkgName)

    #
    # 函数：调用dbus函数
    #
    def call_install_pkgs(self, pkgNames):
        self.dbusDaemon.install(pkgNames)

    #
    # 函数：调用dbus函数
    #
    def call_insone(self, pkgName):
        self.dbusDaemon.insone(pkgName)

    #
    # 函数：调用dbus函数
    #
    def call_upgrade_pkgs(self, pkgNames):
        self.dbusDaemon.upgrade(pkgNames)

    #
    # 函数：调用dbus函数
    #
    def call_upone(self, pkgName):
        self.dbusDaemon.upone(pkgName)

    #
    # 函数：调用dbus函数
    #
    def call_fix_conffile(self):
        self.dbusDaemon.fix_conffile_by_shell("#fix_conffile")

    #
    # 函数：调用dbus函数
    #
    def change_source_list_to_default(self):
        self.dbusDaemon.change_source_list_to_default()

    #
    # 函数：调用dbus函数
    #
    def change_source_list_to_kylin_update(self, lines):
        self.dbusDaemon.change_source_list_to_kylin_update_server(lines)
    #
    #函数：杀进程
    #
    def call_kill_wget(self):
        self.dbusDaemon.kill_pid()

    #
    #函数：调用dbus函数
    #
    def call_copy_deb(self,src_list,appname):
        # print("start_copy")
        self.dbusDaemon.copy_file_to_install(src_list,appname)

    #
    # 函数：调用dbus函数
    #
    def call_get_depends_pkgs(self,appname):
        readlines = self.dbusDaemon.get_depends_pkgs(appname)
        return readlines

    #
    #函数：调用dbus函数
    #
    def call_cancel_download(self,appname):
        self.dbusDaemon.cancel_download_app(appname)

    #
    #函数：调用dbus函数
    #
    def call_down_dep_pkg(self,appname):
        self.dbusDaemon.down_dep_pkg(appname)

    #
    #函数：处理down_dep_finish信号
    #
    def slot_down_dep_finish(self,appname):
        self.down_dep_finish.emit(appname)


    #
    # 函数：调用dbus 开机启动开启函数
    #
    def call_autostart_open(self):
        self.dbusDaemon.autostart_open()

    #
    # 函数：调用dbus 开机启动开启函数
    #
    def call_modify_conf(self, path, group, key, value):
        self.dbusDaemon.modify_conf(path, group, key, value)

    #
    # 函数：调用dbus 开机启动关闭函数
    #
    def call_autostart_close(self):
        self.dbusDaemon.autostart_close()

    #
    # 函数：调用dbus 每日定时启动开启函数
    #
    def call_dailystart_open(self):
        self.dbusDaemon.dailystart_open()

    #
    # 函数：调用dbus 每日定时启动关闭函数
    #
    def call_dailystart_close(self):
        self.dbusDaemon.dailystart_close()

    #
    # 函数：调用dbus 安装本地deb包
    #
    def call_install_debfile(self, path):
        self.dbusDaemon.install_debfile(path)

    #
    # 函数：调用dbus 安装本地deb包
    #
    def call_change_internet_domain_name(self, ip):
        # print("ip:",ip)
        self.dbusDaemon.change_intranet_domain_name(ip)
        # print(ip)

    #
    # 函数：调用源管理器的dbus接口更新模板
    #
    def call_update_source_temple(self):
        return self.sourceDaemon.updateSourceTemplate()

    #
    # 函数：调用源管理器的dbus接口更新源
    #
    def call_update_source_packages(self):
        msg = self.iface.call("updateSourcePackages", "kylin-update-manager")
        return msg.arguments().pop(0)
        # return self.sourceDaemon.updateSourcePackages('kylin-update-manager')

    #
    # 函数：检查源管理的dbus服务器是否存在
    #
    def check_software_properties_exists(self):
        if self.sourceDaemon == None:
            return False
        else:
            return True

    #
    # 函数：下载信息槽
    # processtype：信息类型
    # kwarg：信息体
    #
    def slot_kum_fetch_signal(self, processtype, kwarg):
        signalType = "fetch"
        appName = kwarg['download_appname']
        percent = float(kwarg['download_percent'])
        action = kwarg['action']
        processType = processtype

        if (processType == "down_start"):
            self.down_pulse.emit(kwarg["download_percent"], kwarg["download_pkgname"])
            self.down_start.emit(kwarg["download_percent"], kwarg["download_pkgname"])

        if (processType == "down_pulse"):
            self.down_pulse.emit(kwarg["download_percent"], kwarg["download_pkgname"])
            # processType = processType + " 共" + str(int(kwarg['total_bytes'])/1024) + "KB，已下载" + str(int(kwarg['download_bytes'])/1024) + "KB，总个数为" + str(int(kwarg['total_items'])) + "个，正在下载第" + str(int(kwarg['download_items'])) + "个"
            processType = processType + _(" total") + str(int(kwarg['total_bytes']) / 1024) + _("KB, downloaded") + str(
                int(kwarg['download_bytes']) / 1024) + _("KB,totality nums") + str(int(kwarg['total_items'])) + _(
                ", downloading nums") + str(int(kwarg['download_items'])) + _(".")

        if (processType == "down_cancel"):
            signalType = "cancel"
            percent = -1

        if (processType == "down_done"):
            self.down_finish.emit(True, kwarg["download_percent"], kwarg["download_pkgname"])
        self.dbus_process_msg.emit(signalType, appName, action, percent, processType, '')
        # self.emit(self.dbus_process_msg, signalType, appName, action, percent, processType, '')

    #
    # 函数：执行信息槽
    # processtype：信息类型
    # kwarg：信息体
    #
    def slot_kum_apt_signal(self, processtype, kwarg):
        signalType = "apt"
        appName = kwarg['apt_appname']
        percent = float(kwarg['apt_percent'])
        action = kwarg['action']
        processType = processtype
        status = ""
        if (processtype == "status_change" or kwarg['apt_percent'] == "-99"):
            status = kwarg['status']
        # if (processType == "apt_start"):
        #     self.apt_start.emit(kwarg['apt_percent'], appName)
        #
        # if (processType == 'apt_pulse'):
        #     self.apt_start.emit(kwarg['apt_percent'], appName)
        # if (processType == "apt_finish"):
        #         self.install_num += 1
        #         if (str(self.install_num) == kwarg['install_num']):
        #             appName = '#install_and_upgrade'
        #             self.install_num = 0
        #     self.apt_finish.emit(1, kwarg["apt_pkgname"])
        self.dbus_process_msg.emit(signalType, appName, action, percent, processType, status)

    #
    #函数：处理deb复制完成
    #
    def slot_copy_finish(self,appname):
        self.deb_copy_finish.emit(appname)