#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
# reload(sys)
# sys.setdefaultencoding('utf8')
#from gevent import monkey;monkey.patch_all()
from PyQt5.QtCore import QThread
from PyQt5.QtCore import pyqtSignal
from service.db_service import DBService
from model.upackage import UPackage
from utils.enums import Signals, AppType, AptFunc,iname_list,cname_list
import locale
import apt
import apt_pkg
import time
import threading
import  os
import re
from itertools import groupby

class AccessThread(QThread, Signals):

    accessOver = pyqtSignal(int, int, list)
    oneAppMsg = pyqtSignal(dict)
    # get_kylin_update_over = pyqtSignal(name="get_kylin_update_over")
    work_list = []

    db = None
    ca = None
    mutex = None
    quit_thread = True

    def __init__(self, parent=None):
        QThread.__init__(self)

        locale.setlocale(locale.LC_ALL, "zh_CN.UTF-8")
        self.kus = parent
        self.mutex = threading.RLock()
        self.db = DBService()
        self.kus.desktop_or_server = self.kus.mw.cu.desktop_or_server()
        self.kus.os_version = self.kus.mw.cu.get_os_version()
        self.kus.os_codename = self.kus.mw.cu.get_os_codename()
        self.kus.init_conf()
        #self.thread_work = Thread_work(self)
        #self.kus.mw.pre_work()

    #
    # 函数：初始化apt cache，如不在线程中执行则会卡住主线程
    #
    def init_apt_cache(self):
        try:
            self.ca = apt.Cache()
        except Exception as e:
            print("初始化cache失败:", e)

    #
    # 函数：获取kylin-update-*包
    #
    def get_kylin_update_pkg(self):
        ku_list = []
        kum = None

        self.ca.open()
        try:
            pkg = self.ca["kylin-update-desktop"]
            ku_list.append(pkg)
        except:
            try:
                pkg = self.ca["kylin-update-server"]
                ku_list.append(pkg)
            except:
                pass
        try:
            pkg = self.ca["kylin-update-manager"]
            kum = pkg
        except:
            pass
        self.get_kylin_update_over.emit(ku_list, kum)
        # self.emit(self.get_kylin_update_over, ku_list, kum)

    #
    # 函数：判断当前系统是否为龙芯4000，以及安装对应低版本的龙芯内核虚包
    #
    def check_loongson_3A4000(self):
        results = os.popen("lscpu | grep 'Loongson-3A4000'").readlines()
        # print("results:", results)
        if len(results) != 0:
            return True
        else:
            return False

    #
    # 函数：某个包是否已安装
    #
    def check_is_installed(self, appname):
        results = os.popen("dpkg -l | grep " + appname).readlines()
        # print("results:", results)
        if len(results) != 0:
            return True
        else:
            return False

    #
    # 函数：获取更新信息
    #
    def get_all_update(self):
        ordinary_list = []
        important_list = []
        crucial_list = []
        self.ca.close()
        self.ca.open()
        # 需要进行龙芯4000的判断，并根据不同类型装不同的内核基础虚包
        self.loongson = self.check_loongson_3A4000()
        for pkg_name in cname_list:
            try:
                pkg = self.ca[pkg_name]
                # 关键升级
                if(pkg.is_installed == False or pkg.is_upgradable == True):
                    crucial_list.append(pkg.name)
            # 重要升级
            except:
                pass
        for pkg_name in iname_list:
            try:
                pkg = self.ca[pkg_name]
                # if(pkg.name in iname_list):
                if (pkg.name == "kylin-update-patch"):
                    if pkg.is_installed == False or pkg.is_upgradable:
                    #     continue
                    # system_patch_version = self.kus.get_patch_version()
                    # source_patch_version = pkg.candidate.version.upper().split("-")[2]
                    # if (int(system_patch_version) < int(source_patch_version)):
                        upkg = UPackage(pkg, self.ca)
                        self.fill_info_cn(upkg)
                        important_list.append(upkg)
                        self.oneAppMsg.emit({"apptype": AppType.IMPORTANT, "displayName": upkg.displayName,
                                             "name": upkg.name, "icon": upkg.icon,
                                             "downloadSizeHuman": upkg.downloadSizeHuman,
                                             "downloadSize": upkg.downloadSize,
                                             "installedVersion": upkg.installedVersion,
                                             "architecture": pkg._pkg.architecture,
                                             "candidateVersion": upkg.candidateVersion,
                                             "description": upkg.description, "fullversion": ""})
                        # 避免ui卡顿
                        time.sleep(0.05)
                        continue
                    else:
                        continue
                if (pkg.is_installed == False or pkg.is_upgradable == True):
                    # 检测是否是龙芯4000，根据轻情况选择不同的内核包
                    # print(self.loongson, pkg.name)
                    if self.loongson:
                        if pkg.name == "kylin-update-desktop-kernel":
                            # iname_list.remove(pkg.name)
                            continue
                    upkg = UPackage(pkg, self.ca)
                    # print(upkg.fullname)
                    self.fill_info_cn(upkg)
                    important_list.append(upkg)
                    self.oneAppMsg.emit({"apptype": AppType.IMPORTANT, "displayName": upkg.displayName,
                                         "name": upkg.name, "icon": upkg.icon,
                                         "downloadSizeHuman": upkg.downloadSizeHuman,
                                         "downloadSize": upkg.downloadSize, "installedVersion": upkg.installedVersion,
                                         "architecture": pkg._pkg.architecture,
                                         "candidateVersion": upkg.candidateVersion,
                                         "description": upkg.description, "fullversion": ""})
                    # 避免ui卡顿
                    time.sleep(0.05)
                    # 普通升级
                else:
                    pass
            except:
                pass
        # print(len(ordinary_list), len(important_list), crucial_list)
        self.accessOver.emit(len(ordinary_list), len(important_list), crucial_list)

    #
    # 函数：填充中文信息
    #
    def fill_info_cn(self, upkg):
        displayNameCN = self.db.get_display_name_cn_by_name(upkg.name)
        if(displayNameCN != "" and displayNameCN != " " and displayNameCN != "None" and displayNameCN != None):
            upkg.displayName = displayNameCN


    #
    # 函数：线程执行函数
    #
    def run(self):
        self.kus.mw.pre_work()
        #self.thread_work.start()
        while(True):
            # print("self.work_list:",self.work_list)
            if len(self.work_list) == 0:
                time.sleep(0.2)
                continue

            self.mutex.acquire()
            item = self.work_list.pop(0)
            self.mutex.release()


            if(item == AptFunc.INIT):
                self.init_apt_cache()
            elif(item == AptFunc.GET_KYLIN_UPDATE):
                self.get_kylin_update_pkg()
            elif(item == AptFunc.GET_ALL_UPDATE):
                self.get_all_update()

            time.sleep(0.2)


